@@ -137,16 +137,32 @@ Sources:
 - https://adaptivepatchwork.com/2012/03/01/mind-the-end-of-your-line/
 """
 
+__all__ = [
+    "CRLF",
+    "LF",
+    "BlobNormalizer",
+    "LineEndingFilter",
+    "TreeBlobNormalizer",
+    "check_safecrlf",
+    "convert_crlf_to_lf",
+    "convert_lf_to_crlf",
+    "get_clean_filter",
+    "get_clean_filter_autocrlf",
+    "get_smudge_filter",
+    "get_smudge_filter_autocrlf",
+    "normalize_blob",
+]
+
 import logging
-from typing import TYPE_CHECKING, Any, Callable, Optional, Union
+from collections.abc import Callable, Mapping
+from typing import TYPE_CHECKING, Any
 
 if TYPE_CHECKING:
     from .config import StackedConfig
     from .object_store import BaseObjectStore
 
-from . import replace_me
 from .attrs import GitAttributes, Pattern
-from .filters import FilterBlobNormalizer, FilterDriver, FilterRegistry
+from .filters import FilterBlobNormalizer, FilterContext, FilterDriver, FilterRegistry
 from .object_store import iter_tree_contents
 from .objects import Blob, ObjectID
 from .patch import is_binary
@@ -162,8 +178,8 @@ class LineEndingFilter(FilterDriver):
 
     def __init__(
         self,
-        clean_conversion: Optional[Callable[[bytes], bytes]] = None,
-        smudge_conversion: Optional[Callable[[bytes], bytes]] = None,
+        clean_conversion: Callable[[bytes], bytes] | None = None,
+        smudge_conversion: Callable[[bytes], bytes] | None = None,
         binary_detection: bool = True,
         safecrlf: bytes = b"false",
     ):
@@ -175,7 +191,7 @@ class LineEndingFilter(FilterDriver):
 
     @classmethod
     def from_config(
-        cls, config: Optional["StackedConfig"], for_text_attr: bool = False
+        cls, config: "StackedConfig | None", for_text_attr: bool = False
     ) -> "LineEndingFilter":
         """Create a LineEndingFilter from git configuration.
 
@@ -236,7 +252,7 @@ class LineEndingFilter(FilterDriver):
             # For text attribute: always normalize to LF on checkin
             # Smudge behavior depends on core.eol and core.autocrlf
             smudge_filter = get_smudge_filter(core_eol, autocrlf)
-            clean_filter: Optional[Callable[[bytes], bytes]] = convert_crlf_to_lf
+            clean_filter: Callable[[bytes], bytes] | None = convert_crlf_to_lf
         else:
             # Normal autocrlf behavior
             smudge_filter = get_smudge_filter(core_eol, autocrlf)
@@ -287,7 +303,7 @@ class LineEndingFilter(FilterDriver):
         """Clean up any resources held by this filter driver."""
         # LineEndingFilter doesn't hold any resources that need cleanup
 
-    def reuse(self, config, filter_name: str) -> bool:
+    def reuse(self, config: "StackedConfig", filter_name: str) -> bool:
         """Check if this filter driver should be reused with the given configuration."""
         # LineEndingFilter is lightweight and should always be recreated
         # to ensure it uses the latest configuration
@@ -372,7 +388,7 @@ def check_safecrlf(
 
 def get_smudge_filter(
     core_eol: str, core_autocrlf: bytes
-) -> Optional[Callable[[bytes], bytes]]:
+) -> Callable[[bytes], bytes] | None:
     """Returns the correct smudge filter based on the passed arguments."""
     # Git attributes handling is done by the filter infrastructure
     return get_smudge_filter_autocrlf(core_autocrlf)
@@ -380,7 +396,7 @@ def get_smudge_filter(
 
 def get_clean_filter(
     core_eol: str, core_autocrlf: bytes
-) -> Optional[Callable[[bytes], bytes]]:
+) -> Callable[[bytes], bytes] | None:
     """Returns the correct clean filter based on the passed arguments."""
     # Git attributes handling is done by the filter infrastructure
     return get_clean_filter_autocrlf(core_autocrlf)
@@ -388,7 +404,7 @@ def get_clean_filter(
 
 def get_smudge_filter_autocrlf(
     core_autocrlf: bytes,
-) -> Optional[Callable[[bytes], bytes]]:
+) -> Callable[[bytes], bytes] | None:
     """Returns the correct smudge filter base on autocrlf value.
 
     Args:
@@ -405,7 +421,7 @@ def get_smudge_filter_autocrlf(
 
 def get_clean_filter_autocrlf(
     core_autocrlf: bytes,
-) -> Optional[Callable[[bytes], bytes]]:
+) -> Callable[[bytes], bytes] | None:
     """Returns the correct clean filter base on autocrlf value.
 
     Args:
@@ -421,57 +437,6 @@ def get_clean_filter_autocrlf(
     return None
 
 
-# Backwards compatibility wrappers
-@replace_me(since="0.23.1", remove_in="0.25.0")
-def get_checkout_filter(
-    core_eol: str, core_autocrlf: Union[bool, str], git_attributes: dict[str, Any]
-) -> Optional[Callable[[bytes], bytes]]:
-    """Deprecated: Use get_smudge_filter instead."""
-    # Convert core_autocrlf to bytes for compatibility
-    if isinstance(core_autocrlf, bool):
-        autocrlf_bytes = b"true" if core_autocrlf else b"false"
-    else:
-        autocrlf_bytes = (
-            core_autocrlf.encode("utf-8")
-            if isinstance(core_autocrlf, str)
-            else core_autocrlf
-        )
-    return get_smudge_filter(core_eol, autocrlf_bytes)
-
-
-@replace_me(since="0.23.1", remove_in="0.25.0")
-def get_checkin_filter(
-    core_eol: str, core_autocrlf: Union[bool, str], git_attributes: dict[str, Any]
-) -> Optional[Callable[[bytes], bytes]]:
-    """Deprecated: Use get_clean_filter instead."""
-    # Convert core_autocrlf to bytes for compatibility
-    if isinstance(core_autocrlf, bool):
-        autocrlf_bytes = b"true" if core_autocrlf else b"false"
-    else:
-        autocrlf_bytes = (
-            core_autocrlf.encode("utf-8")
-            if isinstance(core_autocrlf, str)
-            else core_autocrlf
-        )
-    return get_clean_filter(core_eol, autocrlf_bytes)
-
-
-@replace_me(since="0.23.1", remove_in="0.25.0")
-def get_checkout_filter_autocrlf(
-    core_autocrlf: bytes,
-) -> Optional[Callable[[bytes], bytes]]:
-    """Deprecated: Use get_smudge_filter_autocrlf instead."""
-    return get_smudge_filter_autocrlf(core_autocrlf)
-
-
-@replace_me(since="0.23.1", remove_in="0.25.0")
-def get_checkin_filter_autocrlf(
-    core_autocrlf: bytes,
-) -> Optional[Callable[[bytes], bytes]]:
-    """Deprecated: Use get_clean_filter_autocrlf instead."""
-    return get_clean_filter_autocrlf(core_autocrlf)
-
-
 class BlobNormalizer(FilterBlobNormalizer):
     """An object to store computation result of which filter to apply based on configuration, gitattributes, path and operation (checkin or checkout).
 
@@ -481,7 +446,7 @@ class BlobNormalizer(FilterBlobNormalizer):
     def __init__(
         self,
         config_stack: "StackedConfig",
-        gitattributes: dict[str, Any],
+        gitattributes: Mapping[str, Any],
         core_eol: str = "native",
         autocrlf: bytes = b"false",
         safecrlf: bytes = b"false",
@@ -516,9 +481,12 @@ class BlobNormalizer(FilterBlobNormalizer):
 
         git_attributes = GitAttributes(git_attrs_patterns)
 
+        # Create FilterContext for parent class
+        filter_context = FilterContext(filter_registry)
+
         # Initialize parent class with gitattributes
         # The filter infrastructure will handle gitattributes processing
-        super().__init__(config_stack, git_attributes, filter_registry)
+        super().__init__(config_stack, git_attributes, filter_context=filter_context)
 
         # Store original filters for backward compatibility
         self.fallback_read_filter = smudge_filter
@@ -634,9 +602,9 @@ class TreeBlobNormalizer(BlobNormalizer):
     def __init__(
         self,
         config_stack: "StackedConfig",
-        git_attributes: dict[str, Any],
+        git_attributes: Mapping[str, Any],
         object_store: "BaseObjectStore",
-        tree: Optional[ObjectID] = None,
+        tree: ObjectID | None = None,
         core_eol: str = "native",
         autocrlf: bytes = b"false",
         safecrlf: bytes = b"false",
