@@ -234,7 +234,7 @@ def check_for_daemon(limit=10, delay=0.1, timeout=0.1, port=TCP_GIT_PORT) -> boo
         try:
             s.connect(("localhost", port))
             return True
-        except socket.timeout:
+        except TimeoutError:
             pass
         except OSError as e:
             if getattr(e, "errno", False) and e.errno != errno.ECONNREFUSED:
