@@ -19,12 +19,12 @@
 # License, Version 2.0.
 #
 
+import unittest
 
-def test_suite():
-    import unittest
 
+def test_suite() -> unittest.TestSuite:
     names = [
-        "diffstat",
+        "greenthreads",
         "paramiko_vendor",
         "release_robot",
         "swift",
