@@ -22,8 +22,6 @@
 """Tests for dulwich.rebase."""
 
 import importlib.util
-import os
-import tempfile
 
 from dulwich.objects import Blob, Commit, Tree
 from dulwich.rebase import (
@@ -36,7 +34,7 @@ from dulwich.rebase import (
     rebase,
     start_interactive,
 )
-from dulwich.repo import MemoryRepo, Repo
+from dulwich.repo import MemoryRepo
 from dulwich.tests.utils import make_commit
 
 from . import DependencyMissing, TestCase
@@ -49,6 +47,7 @@ class RebaserTestCase(TestCase):
         """Set up test repository."""
         super().setUp()
         self.repo = MemoryRepo()
+        self.addCleanup(self.repo.close)
 
     def _setup_initial_commit(self):
         """Set up initial commit for tests."""
@@ -387,90 +386,6 @@ class RebaserTestCase(TestCase):
         self.assertNotIn(b"a.txt", new_c_tree)
 
 
-class RebasePorcelainTestCase(TestCase):
-    """Tests for the porcelain rebase function."""
-
-    def setUp(self):
-        """Set up test repository."""
-        super().setUp()
-        self.test_dir = tempfile.mkdtemp()
-        self.repo = Repo.init(self.test_dir)
-
-        # Create initial commit
-        with open(os.path.join(self.test_dir, "README.md"), "wb") as f:
-            f.write(b"# Test Repository\n")
-
-        self.repo.get_worktree().stage(["README.md"])
-        self.initial_commit = self.repo.get_worktree().commit(
-            message=b"Initial commit",
-            committer=b"Test User <test@example.com>",
-            author=b"Test User <test@example.com>",
-        )
-
-    def tearDown(self):
-        """Clean up test directory."""
-        import shutil
-
-        shutil.rmtree(self.test_dir)
-
-    def test_porcelain_rebase(self):
-        """Test rebase through porcelain interface."""
-        from dulwich import porcelain
-
-        # Create and checkout feature branch
-        self.repo.refs[b"refs/heads/feature"] = self.initial_commit
-        porcelain.checkout(self.repo, "feature")
-
-        # Add commit to feature branch
-        with open(os.path.join(self.test_dir, "feature.txt"), "wb") as f:
-            f.write(b"Feature file\n")
-
-        porcelain.add(self.repo, ["feature.txt"])
-        porcelain.commit(
-            self.repo,
-            message="Add feature",
-            author="Test User <test@example.com>",
-            committer="Test User <test@example.com>",
-        )
-
-        # Switch to main and add different commit
-        porcelain.checkout(self.repo, "master")
-
-        with open(os.path.join(self.test_dir, "main.txt"), "wb") as f:
-            f.write(b"Main file\n")
-
-        porcelain.add(self.repo, ["main.txt"])
-        porcelain.commit(
-            self.repo,
-            message="Main update",
-            author="Test User <test@example.com>",
-            committer="Test User <test@example.com>",
-        )
-
-        # Switch back to feature and rebase
-        porcelain.checkout(self.repo, "feature")
-
-        # Perform rebase
-        new_shas = porcelain.rebase(self.repo, "master")
-
-        # Should have rebased one commit
-        self.assertEqual(len(new_shas), 1)
-
-        # Check that the rebased commit has the correct parent and tree
-        feature_head = self.repo.refs[b"refs/heads/feature"]
-        feature_commit_obj = self.repo[feature_head]
-
-        # Should have master as parent
-        master_head = self.repo.refs[b"refs/heads/master"]
-        self.assertEqual(feature_commit_obj.parents, [master_head])
-
-        # Tree should have both files
-        tree = self.repo[feature_commit_obj.tree]
-        self.assertIn(b"feature.txt", tree)
-        self.assertIn(b"main.txt", tree)
-        self.assertIn(b"README.md", tree)
-
-
 class InteractiveRebaseTestCase(TestCase):
     """Tests for interactive rebase functionality."""
 
@@ -478,6 +393,7 @@ class InteractiveRebaseTestCase(TestCase):
         """Set up test repository."""
         super().setUp()
         self.repo = MemoryRepo()
+        self.addCleanup(self.repo.close)
         self._setup_initial_commit()
 
     def _setup_initial_commit(self):
